from __future__ import unicode_literals

from . import _ellf_ffi
from ._ellf_ffi.lib import *

try:
    from numba import cffi_support as _cffi_support
    _cffi_support.register_module(_ellf_ffi)
except ImportError:
    pass



from __future__ import unicode_literals

from numpy.testing import assert_almost_equal

from ncephes import ellf


def test_ellie():
    assert_almost_equal(ellf.ellie(-5.3, 0.12), -5.12290521194)


def test_ellik():
    assert_almost_equal(ellf.ellik(-5.3, 0.12), -5.48607395126)


def test_ellpe():
    assert_almost_equal(ellf.ellpe(0.12), 1.120741662164857)


def test_ellpk():
    assert_almost_equal(ellf.ellpk(0.12), 2.492635323239716)



from __future__ import unicode_literals

from numpy.testing import assert_almost_equal

from ncephes import cprob


def test_bdtr():
    assert_almost_equal(cprob.bdtr(4, 5, 0.25), 0.9990234375000000)
    assert_almost_equal(cprob.bdtrc(4, 5, 0.25), 0.0009765625000000)
    assert_almost_equal(cprob.bdtri(4, 5, 0.25), 0.9440875112949020)


def test_btdtr():
    assert_almost_equal(cprob.btdtr(4, 5, 0.25), 0.1138153076171875)


def test_chdtr():
    assert_almost_equal(cprob.chdtr(4, 5), 0.7127025048163542)
    assert_almost_equal(cprob.chdtrc(4, 5), 0.2872974951836458)
    assert_almost_equal(cprob.chdtri(4, 0.3), 4.8784329665604087)


def test_expx2():
    assert_almost_equal(cprob.expx2(1, 2), 2.7182818284590451)


def test_fdtr():
    assert_almost_equal(cprob.fdtr(4, 5, 0.3), 0.1333536247071635)
    assert_almost_equal(cprob.fdtrc(4, 5, 0.3), 0.8666463752928364)
    assert_almost_equal(cprob.fdtri(4, 5, 0.3), 1.6286329978911640)


def test_gamma():
    assert_almost_equal(cprob.cephes_gamma(1.5), 0.8862269254527579)
    assert_almost_equal(cprob.lgam(3.4), 1.0923280598027414)


def test_gdtr():
    assert_almost_equal(cprob.gdtr(1, 2, 0.1), 0.0046788401604445)
    assert_almost_equal(cprob.gdtrc(1, 2, 0.1), 0.9953211598395555)


def test_igam():
    assert_almost_equal(cprob.igam(1, 2), 0.8646647167633873)
    assert_almost_equal(cprob.igamc(2, 1), 0.7357588823428847)


def test_incbet():
    assert_almost_equal(cprob.incbet(1., 3., 0.3), 0.657)


def test_incbi():
    assert_almost_equal(cprob.incbi(1., 3., 0.3), 0.1120959982573993)


def test_kolmogorov():
    assert_almost_equal(cprob.smirnov(2, 0.3), 0.61)
    assert_almost_equal(cprob.kolmogorov(2), 0.00067092525578)
    assert_almost_equal(cprob.smirnovi(2, 0.3), 0.474679434488)
    assert_almost_equal(cprob.kolmogi(0.24), 1.02920479826)


def test_nbdtr():
    assert_almost_equal(cprob.nbdtr(1, 3, 0.5), 0.3125)
    assert_almost_equal(cprob.nbdtrc(1, 3, 0.5), 0.6875)
    assert_almost_equal(cprob.nbdtri(1, 3, 0.5), 0.614272431868)


def test_ndtr():
    assert_almost_equal(cprob.ndtr(0.3), 0.617911422189)
    assert_almost_equal(cprob.erf(1.3), 0.9340079449406524)
    assert_almost_equal(cprob.erfc(0.3), 0.671373240541)


def test_ndtri():
    assert_almost_equal(cprob.ndtri(0.6), 0.253347103136)


def test_pdtr():
    assert_almost_equal(cprob.pdtr(2, 0.15), 0.999497137624)
    assert_almost_equal(cprob.pdtrc(2, 0.15), 0.000502862376402)
    assert_almost_equal(cprob.pdtri(2, 0.15), 4.72305156339)


def test_stdtr():
    assert_almost_equal(cprob.stdtr(2, 3), 0.952267016867)
    assert_almost_equal(cprob.stdtri(5, 0.1), -1.47588404882)


def test_unity():
    assert_almost_equal(cprob.log1p(0.1), 0.0953101798043)
    assert_almost_equal(cprob.expm1(0.5), 0.6487212707)
    assert_almost_equal(cprob.cosm1(0.9), -0.378390031729)



from __future__ import unicode_literals

from os.path import splitext
from os.path import basename
from os.path import join

from glob import glob

import re


def get_sources(module):
    sources = glob(join('ncephes', 'cephes', module, '*.c'))
    sources += [join('ncephes', 'cephes', 'cmath', 'isnan.c')]
    sources += [join('ncephes', 'cephes', 'mtherr.c')]
    if module == 'misc':
        sources += [join('ncephes', 'cephes', 'cprob', 'gamma.c')]
        sources += [join('ncephes', 'cephes', 'polyn', 'polyn.c')]
        sources += [join('ncephes', 'cephes', 'cmath', 'powi.c')]
    return sources


def get_include_dirs(module):
    return [join('ncephes', 'cephes', module), join('ncephes', 'cephes')]


def get_fdecls(module):
    from build_helpers import read_export_file
    from build_helpers import fetch_func_decl

    regex = re.compile(r'^.* (.+)\(.*\).*$')
    fdecls = []
    export_table = read_export_file(join('ncephes', 'cephes',
                                         '%s_export.txt' % module))
    for fp in glob(join('ncephes', 'cephes', module, '*.c')):
        modname = splitext(basename(fp))[0]
        if modname in export_table:
            fnames = export_table[modname]
            for fd in fetch_func_decl(fp):
                fdname = regex.match(fd).group(1)
                for fn in fnames:
                    if fn == fdname:
                        fdecls.append(fd + ';')

    return fdecls



from __future__ import unicode_literals

from numpy.testing import assert_almost_equal

from numba import jit

from ncephes import cprob
from ncephes.cprob import incbet


@jit(nogil=True, nopython=True)
def _incbet(a, b, x):
    return incbet(a, b, x)


@jit(nogil=True)
def erf(x):
    return cprob.erf(x)


def test_incbet():
    assert_almost_equal(_incbet(1., 3., 0.3), 0.657)


def test_erf():
    assert_almost_equal(erf(1.3), 0.9340079449406524)



from __future__ import unicode_literals

import sys
PY3 = sys.version_info > (3,)

from os.path import join

from cffi import FFI

from module_info import get_fdecls
from module_info import get_include_dirs
from module_info import get_sources


def _make(module):
    fdecls = get_fdecls(module)

    fdecl_extern = '\n'.join(['extern ' + f for f in fdecls])
    fdecl_noextern = '\n'.join(fdecls)
    cconst = join('ncephes', 'cephes', 'const.c')
    sources = get_sources(module) + [cconst]

    if not PY3:
        fdecl_extern = fdecl_extern.encode('ascii', 'ignore')
        fdecl_noextern = fdecl_noextern.encode('ascii', 'ignore')
        sources = [s.encode('ascii', 'ignore') for s in sources]

    ffi = FFI()
    ffi.set_source('ncephes._%s_ffi' % module,
                   fdecl_extern,
                   include_dirs=get_include_dirs(module),
                   sources=sources,
                   libraries=[],
                   extra_compile_args=['-Ofast'],
                   language='c')
    ffi.cdef(fdecl_noextern)
    return ffi

cprob = _make('cprob')
ellf = _make('ellf')
misc = _make('misc')
polyn = _make('polyn')



from __future__ import unicode_literals

import os
from os.path import join
import sys
PY3 = sys.version_info > (3,)
from setuptools import setup
from setuptools import find_packages

try:
    import pypandoc
    long_description = pypandoc.convert('README.md', 'rst')
except(IOError, ImportError):
    long_description = open('README.md').read()

def get_sources(module):
    from module_info import get_sources
    sources = get_sources(module)
    sources += [join('ncephes', 'cephes', module + '_ffcall.c')]
    sources += [join('ncephes', 'cephes', 'const.c')]
    return sources


def get_include_dirs(module):
    from module_info import get_include_dirs as gid
    return (gid(module) + [join('ncephes', 'include')] +
            [join('ncephes', 'cephes')])


class GetApi(object):

    def __init__(self, module):
        self._module = module

    def __call__(self):
        from build_capi import CApiLib
        module = self._module
        return CApiLib(name='ncephes.lib.' + 'n' + module,
                       sources=get_sources(module),
                       include_dirs=get_include_dirs(module))


def create_capi_libs():
    modules = open('supported_modules.txt').read().split("\n")[:-1]
    return [GetApi(module) for module in modules]


def setup_package():
    src_path = os.path.dirname(os.path.abspath(sys.argv[0]))
    old_path = os.getcwd()
    os.chdir(src_path)
    sys.path.insert(0, src_path)

    needs_pytest = {'pytest', 'test', 'ptr'}.intersection(sys.argv)
    pytest_runner = ['pytest-runner'] if needs_pytest else []

    setup_requires = ['build_capi>=1.1.5', 'cffi>=1.7',
                      'pycparser'] + pytest_runner
    install_requires = ['cffi>=1.7']
    tests_require = []

    modules = open('supported_modules.txt').read().split("\n")[:-1]
    cffi_modules = ['module_build.py:%s' % m for m in modules]

    hdr_dir = join('ncephes', 'include', 'ncephes')
    hdr_files = [join(hdr_dir, m + '.h') for m in modules]

    if PY3:
        package_data={'': [join('ncephes', 'lib', '*.*')]}
    else:
        package_data={b'': [join(b'ncephes', b'lib', b'*.*')]}

    metadata = dict(
        name='ncephes',
        version='1.0.12',
        maintainer="Danilo Horta",
        maintainer_email="danilo.horta@gmail.com",
        description="Python interface for the Cephes library.",
        long_description=long_description,
        license="MIT",
        url='https://github.com/Horta/ncephes',
        packages=find_packages(),
        zip_safe=False,
        cffi_modules=cffi_modules,
        setup_requires=setup_requires,
        install_requires=install_requires,
        tests_require=tests_require,
        classifiers=[
            "Development Status :: 5 - Production/Stable",
            "Intended Audience :: Developers",
            "License :: OSI Approved :: MIT License",
            "Programming Language :: Python :: 2.7",
            "Programming Language :: Python :: 3.4",
            "Programming Language :: Python :: 3.5",
            "Operating System :: OS Independent",
        ],
        capi_libs=create_capi_libs(),
        keywords=["cephes", "math", "numba"],
        include_package_data=True,
        data_files=[(hdr_dir, hdr_files)],
        package_data=package_data,
    )

    try:
        from distutils.command.bdist_conda import CondaDistribution
    except ImportError:
        pass
    else:
        metadata['distclass'] = CondaDistribution
        metadata['conda_buildnum'] = 1
        metadata['conda_features'] = ['mkl']

    try:
        setup(**metadata)
    finally:
        del sys.path[0]
        os.chdir(old_path)

if __name__ == '__main__':
    setup_package()
